import pytz
import ujson

from ..types import GpsdServiceArgs, TzServiceArgs
from ...eventlog import *
from ...model.geolocation import Geolocation
from ...types import mk_error_text, JnodeError
from ...path_helpers import UI_TIME_JNODE_FILEPATH


class JsonGpsdServiceArgs(GpsdServiceArgs):
    def __init__(self, *args, **kwargs):
        GpsdServiceArgs.__init__(self, *args, **kwargs)

    @staticmethod
    def default():
        """
        :rtype: JsonGpsdServiceArgs
        """
        return GpsdServiceArgs(endpoint=('localhost', 2947))

    @staticmethod
    def deserialzie(raw_jnode):
        """
        :type raw_jnode: dict
        :rtype: JsonGpsdServiceArgs
        :raises JnodeError:
        """
        rt_jnode = raw_jnode.get('rt')
        if rt_jnode is None:
            return JsonGpsdServiceArgs.default()

        jnode = rt_jnode.get('source')
        if jnode is None:
            log_d(mk_error_text('#/'))
            raise JnodeError

        if not (type(jnode) in [tuple, list] and len(jnode) == 3):
            log_d(mk_error_text('#/'))
            raise JnodeError

        # deserialize geodata
        if jnode[0] == 'static':
            _, latitude, longitude = jnode
            if not (type(latitude) in [int, float] and -90.0 <= latitude <= +90.0):
                log_d(mk_error_text('#/1'))
                raise JnodeError

            if not (type(longitude) in [int, float] and -180.0 <= longitude <= +180.0):
                log_d(mk_error_text('#/2'))
                raise JnodeError

            return JsonGpsdServiceArgs(geodata=Geolocation(latitude, longitude))

        if jnode[0] == 'gpsd':
            _, gpsd_host, gpsd_port = jnode
            if not (type(gpsd_host) in [str, unicode] and len(gpsd_host) <= 32):
                log_d(mk_error_text('#/1'))
                raise JnodeError

            if not (type(gpsd_port) in [int] and 0 < gpsd_port <= 65535):
                log_d(mk_error_text('#/2'))
                raise JnodeError

            return JsonGpsdServiceArgs(endpoint=(gpsd_host, gpsd_port))

        log_d(mk_error_text('#/0'))
        raise JnodeError


class JsonTzServiceArgs(TzServiceArgs):
    def __init__(self, *args, **kwargs):
        TzServiceArgs.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialzie(jnode):
        """
        :type jnode: dict
        :rtype: JsonTzServiceArgs
        """
        # deserialize #/rw
        rw_jnode = jnode.get('rw')
        if dict != type(rw_jnode):
            log_d(mk_error_text('#/rw'))
            raise JnodeError

        # deserialize #/rw/tz
        tz_name = rw_jnode.get('args', {}).get('tz')
        if not (type(tz_name) in [unicode, str] and 3 <= len(tz_name) <= 32):
            log_d(mk_error_text('#/rw/tz'))
            raise JnodeError

        if tz_name not in pytz.all_timezones:
            log_d(mk_error_text('#/rw/tz'))
            raise JnodeError

        tz_info = pytz.timezone(tz_name)
        return JsonTzServiceArgs(tz_info)

    @staticmethod
    def create(filepath=UI_TIME_JNODE_FILEPATH):
        """
        :type filepath: str
        :rtype: JsonTzServiceArgs
        """
        with open(filepath) as tz_conf_jnode:
            tz_conf_jnode = ujson.load(tz_conf_jnode)

        return JsonTzServiceArgs.deserialzie(tz_conf_jnode)

